const path = require("path");
const multer = require("multer");
const dayjs = require("dayjs");
const { format } = require("date-fns");
const UserModel = require("../models/Users");
const SettingModel = require("../models/Settings");
const HelpModel = require("../models/Helps");
const EmployeeModel = require("../models/Employees");
const FinalclientModel = require("../models/Finalclients");
const AbsencelogModel = require("../models/Absencelogs");
const ReservationModel = require("../models/Reservations");

function getTimeSlots(opening_time, closing_time, duration) {
  const result = [];
  let start24 = dayjs(opening_time, "h:mma").format("HH:mm");
  let end24 = dayjs(closing_time, "h:mma").format("HH:mm");

  const [startHour, startMinute] = start24.split(":").map(Number);
  const [endHour, endMinute] = end24.split(":").map(Number);

  let current = new Date();
  current.setHours(startHour, startMinute, 0, 0); // Set to opening time

  let end = new Date();
  end.setHours(endHour, endMinute, 0, 0); // Set to closing time

  while (current <= end) {
    let formattedTime = dayjs(current).format("hh:mma"); // Format in 12-hour with lowercase am/pm
    result.push(formattedTime);

    current = new Date(current.getTime() + duration * 60 * 1000);
  }

  return result;
}


const sendMail = require("./mailController");

exports.index = async function (req, res) {
  let { companyname } = req.params;

  let account = await UserModel.findOne({ company_name: companyname });
  let setting = await SettingModel.findOne({});

  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  return res.status(200).json({
    status: "true",
    message: "URL Access successfully",
    settings: {
      title:
        account.language == "fr"
          ? setting.title_fr
          : account.language == "ro"
          ? setting.title_ro
          : setting.title,
      description:
        account.language == "fr"
          ? setting.description_fr
          : account.language == "ro"
          ? setting.description_ro
          : setting.description,
    },
    data: account,
  });
};

exports.login = async function (req, res) {
  let { companyname } = req.params;
  let { email, password } = req.body;

  if (!email || !password) {
    return res.status(400).json({
      status: "false",
      message: "Missing required fields",
    });
  }

  let account = await UserModel.findOne({ company_name: companyname });
  console.log(account);
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "Account not found",
    });
  }

  if (account.email === email && account.password === password) {
    return res.status(200).json({
      status: "true",
      message: "URL Access successfully",
      accountdetail: account,
    });
  } else {
    return res.status(404).json({
      status: "false",
      message: "Credentials not match",
    });
  }
};

exports.register = async function (req, res) {
  let { companyname } = req.params;
  let { email, password } = req.body;

  if (!email || !password) {
    return res.status(400).json({
      status: "false",
      message: "Missing required fields",
    });
  }

  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "Account not found",
    });
  }

  if (account.email === email && account.password === password) {
    return res.status(200).json({
      status: "true",
      message: "URL Access successfully",
      accountdetail: account,
    });
  } else {
    return res.status(404).json({
      status: "false",
      message: "Credentials not match",
    });
  }
};

exports.profile = async function (req, res) {
  let { user_id } = req.body;
  if (!user_id) {
    return res.status(404).json({
      status: "false",
      message: "user_id Missing",
    });
  }
  let account = await UserModel.findOne({ _id: user_id });
  let setting = await SettingModel.findOne();

  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "Account not found",
    });
  }

  if (req.body.opening_time) {
    let opening = req.body.opening_time;
    account.opening_time = opening.toLowerCase();
  }

  if (req.body.closing_time) {
    let closing = req.body.closing_time;
    account.closing_time = closing.toLowerCase();
  }

  if (req.body.duration) {
    account.duration = req.body.duration;
  }
  if (req.body.hasOwnProperty('automatic_booking')) {
    account.automatic_booking = (req.body.automatic_booking) ? 1 : 0;
  }
  await account.save();
  console.log('account',account);
  console.log('autoboo',account.automatic_booking)
  return res.status(200).json({
    status: "true",
    message: "Profile Updated Sucessfully!",
    accountdetail: account,
    settings: setting,
    image_url : path.join(__dirname, process.env.ASSET_PATH + "/users/")
  });
};

exports.subscripe_payment = async function (req, res) {
  try {
    // Assume the client sends the amount
    const { amount } = req.body; // Amount in cents (e.g., $10 is 1000 cents)

    const paymentIntent = await stripe.paymentIntents.create({
      amount: amount,
      currency: "usd",
      payment_method_types: ["card"],
    });

    res.json({
      clientSecret: paymentIntent.client_secret,
    });
  } catch (error) {
    console.error(error);
    res.status(500).send({ error: error.message });
  }
};

exports.addEmployee = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }

  let { name } = req.body;

  if (!name) {
    return res.status(404).json({
      status: "false",
      message: "Missing required Params",
    });
  }
  req.body.business_owner = account._id;
  req.body.company_name = companyname;
  const newAccount = new EmployeeModel(req.body);

  try {
    const accountdetails = await newAccount.save();
    return res.status(201).json({
      status: "true",
      data: accountdetails,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.removeEmployee = async function (req, res) {
  let { id } = req.body;
  if (!id) {
    return res.status(404).json({
      status: "false",
      message: "Missing required Params",
    });
  }
  console.log('cakled',id)
  const formattedDate = format(new Date(Date.now()), "yyyy-MM-dd");
  try {
    const updatedAccount = await EmployeeModel.findByIdAndUpdate(
      id,
      { status: 2, deleted_at: formattedDate },
      { new: true } // Return the updated document
    );

    if (!updatedAccount) {
      return res.status(404).json({
        status: "false",
        message: "Employee not found",
      });
    }

    return res.status(200).json({
      status: "true",
      message: "Employee removed successfully",
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.editEmployee = async function (req, res) {
  let { id } = req.body;
  if (!id) {
    return res.status(404).json({
      status: "false",
      message: "Missing required Params",
    });
  }
  try {
    const updatedAccount = await EmployeeModel.findByIdAndUpdate(
      req.body.id,
      { name: req.body.name },
      { new: true } // Return the updated document
    );

    if (!updatedAccount) {
      return res.status(404).json({
        status: "false",
        message: "Employee not found",
      });
    }

    return res.status(200).json({
      status: "true",
      message: "Employee edited successfully",
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.exployees = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  try {
    const records = await EmployeeModel.find({ business_owner: account._id , status :{$ne : 2}});
    if (!records) {
      return res.status(404).json({
        status: "false",
        message: "Employee not found",
      });
    }

    return res.status(200).json({
      status: "true",
      data: records,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};
exports.addabsence = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });

  let { employee } = req.body;
  if (!employee) {
    return res.status(500).json({
      status: "false",
      message: "Required Param missing",
    });
  }
  req.body.date = format(new Date(req.body.date), "yyyy-MM-dd");
  req.body.business_owner = account._id;
  req.body.company_name = account.company_name;
  const startTime24 = dayjs(req.body.start_time, "h:mma").format("HH:mm");
  const endTime24 = dayjs(req.body.end_time, "h:mma").format("HH:mm");
  req.body.end_time24 = endTime24;
  req.body.start_time24 = startTime24;
  let log = new AbsencelogModel(req.body);
  await log.save();
  return res.status(200).json({
    status: "true",
    message: "Absence added successfully!",
  });
};

exports.getAbsencelog = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  const records = await EmployeeModel.find({ business_owner: account._id,status :{$ne : 2} });

  let absencehistory = await AbsencelogModel.find({
    business_owner: account._id,
  }).populate("employee");
  // if (absencehistory.length > 0) {
  return res.status(200).json({
    status: "true",
    data: absencehistory,
    employees: records,
  });
  // } else {
  // return res.status(404).json({
  //   status: "false",
  //   message: "Absence history not found",
  // });
  // }
};

exports.addFinalclient = async function (req, res) {
  let { companyname } = req.params;

  let account = await UserModel.findOne({ company_name: companyname });

  let { name, email, phone_number } = req.body;

  if (!name || !email || !phone_number) {
    return res.status(404).json({
      status: "false",
      message: "Missing required Params",
    });
  }
  req.body.business_owner = account._id;
  req.body.company_name = account.company_name;
  req.body.password = "123456";
  req.body.created_at = Date.now();

  const newAccount = new FinalclientModel(req.body);

  try {
    const accountdetails = await newAccount.save();
    return res.status(201).json({
      status: "true",
      message: "Final client successfully created!",
      data: accountdetails,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.manageFinalclients = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  let finalclients = await FinalclientModel.find({
    business_owner: account._id,
  });
  if (finalclients.length > 0) {
    return res.status(201).json({
      status: "true",
      data: finalclients,
    });
  } else {
    return res.status(500).json({
      status: "false",
      message: "No records found!",
    });
  }
};

exports.getRequirements = async function (req, res) {
  let { companyname } = req.params;

  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      data: [],
    });
  } else {
    return res.status(201).json({
      status: "true",
      data: account.requirements,
    });
  }
};

exports.addRequirements = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  console.log(req.body.requirements);
  account.requirements = req.body.requirements;
  await account.save();
  return res.status(201).json({
    status: "true",
    message: "Changes Updated!",
  });
};

exports.allBookings = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  try {
    let { date } = req.body;
    if (!date) {
      return res.status(404).json({
        status: "false",
        message: "Missing Parameters",
      });
    }

    let given_date = format(new Date(Date.now()), "yyyy-MM-dd");
    if (req.body.date) {
      given_date = req.body.date;
    }

    const records = await EmployeeModel.find({
      business_owner: account._id,
      $or: [{ deleted_at: null }, { deleted_at: { $gt: given_date } }],
    });
    let timeSlots = getTimeSlots("08:00am", "10:00pm", 30);
    if(account.opening_time !="" && account.closing_time)
      timeSlots = getTimeSlots(account.opening_time, account.closing_time, account.duration);

    let result = [];
    for (let index = 0; index < records.length; index++) {
      let absencehistory = await AbsencelogModel.find({
        business_owner: account._id,
        date: date,
        employee: records[index]._id,
      });
      let reservations = await ReservationModel.find({
        business_owner: account._id,
        reservation_date: date,
        employee: records[index]._id,
        status: { $in: ["Pending", "Accepted"] },
      })
        .populate("business_owner")
        .populate("final_client")
        .populate("employee");
      result.push({
        name: records[index].name,
        id: records[index]._id,
        absence: absencehistory,
        bookings: reservations,
      });
    }
    res.status(200).json({
      status: "true",
      data: result,
      opening_time: account.opening_time,
      closing_time: account.closing_time,
      duration : account.duration,
      slots: timeSlots
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Server error" });
  }
};

exports.changeBookingstatus = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }

  try {
    let { booking_id, status } = req.body;
    let updatedData = {};
    updatedData.status = status;
    if(status == "Cancelled")
      updatedData.cancelled_by ="businessowner"
    const changestatus = await ReservationModel.findByIdAndUpdate(
      booking_id,
      updatedData,
      { new: true } // Return the updated document
    );
    res.status(200).json({
      status: "true",
      data: changestatus,
      message : status == "Cancelled" ? "Booking cancelled successfully!" : "Booking approved successfully!"
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Server error" });
  }
};
