const path = require("path");
const multer = require("multer");
const UserModel = require("../models/Users");
const HelpModel = require("../models/Helps");
const SettingModel = require("../models/Settings");
const ReservationModel = require("../models/Reservations");
const EmployeeModel = require("../models/Employees");
const AbsenceModel = require("../models/Absencelogs");
const dayjs = require("dayjs");
const { format, isBefore, startOfToday, parseISO } = require("date-fns");
const customParseFormat = require("dayjs/plugin/customParseFormat");
const sendMail = require("./mailController");
dayjs.extend(customParseFormat);

function addMinutesToTime(start_time, minutesToAdd) {
  const regex = /(\d{1,2}):(\d{2})(am|pm)/i;
  const match = start_time.match(regex);

  if (!match) {
    console.log("Invalid time format");
    return;
  }

  let [_, hour, minute, period] = match;
  hour = parseInt(hour);
  minute = parseInt(minute);

  if (period.toLowerCase() === "pm" && hour !== 12) hour += 12;
  if (period.toLowerCase() === "am" && hour === 12) hour = 0;

  let date = new Date(1970, 0, 1, hour, minute);

  date.setMinutes(date.getMinutes() + minutesToAdd);

  let newHour = date.getHours();
  let newMinute = date.getMinutes();
  let newPeriod = newHour >= 12 ? "pm" : "am";

  newHour = newHour % 12 || 12;
  newMinute = newMinute < 10 ? "0" + newMinute : newMinute;

  return `${newHour}:${newMinute}${newPeriod}`;
}
function getTimeSlots(opening_time, closing_time, duration) {
  const result = [];
  let start24 = dayjs(opening_time, "h:mma").format("HH:mm");
  let end24 = dayjs(closing_time, "h:mma").format("HH:mm");

  const [startHour, startMinute] = start24.split(":").map(Number);
  const [endHour, endMinute] = end24.split(":").map(Number);

  let current = new Date();
  current.setHours(startHour, startMinute, 0, 0); // Set to opening time

  let end = new Date();
  end.setHours(endHour, endMinute, 0, 0); // Set to closing time

  while (current <= end) {
    let formattedTime = dayjs(current).format("hh:mma"); // Format in 12-hour with lowercase am/pm
    result.push(formattedTime);

    current = new Date(current.getTime() + duration * 60 * 1000);
  }

  return result;
}

exports.onBooking = async function (req, res) {
  let { companyname } = req.params;
  let owner = await UserModel.findOne({ company_name: companyname });
  if (!owner) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }

  let { final_client, employee, reservation_date, start_time } = req.body;

  if (!final_client || !reservation_date || !start_time) {
    return res.status(400).json({
      status: "false",
      message: "Required parameters are missing",
    });
  }
  reservation_date = format(new Date(reservation_date), "yyyy-MM-dd");
  const parsedDate = parseISO(reservation_date);
  console.log('parsedDate',parsedDate);
  if (isBefore(parsedDate, startOfToday())) {
    return res.status(200).json({
        status: "false",
        message: "Cannot book for a past date.",
      });
   }
  req.body.reservation_date = reservation_date;
  let end_time = addMinutesToTime(start_time, owner.duration);
  req.body.business_owner = owner._id;
  req.body.end_time = end_time;
  req.body.status = owner.automatic_booking == "1" ? "Accepted" : "Pending";
  req.body.booking_date = Date.now();
  const startTime24 = dayjs(start_time, "h:mma").format("HH:mm");
  const endTime24 = dayjs(end_time, "h:mma").format("HH:mm");
  req.body.end_time24 = endTime24;
  req.body.start_time24 = startTime24;
  console.log("calendar req.body", req.body);
  try {
    
    // check availability
    if (employee) {
      const absentEmployees = await AbsenceModel.countDocuments({
        employee: employee,
        date: reservation_date,
        $or: [
          {
            $and: [
              { start_time24: { $gt: startTime24 } },
              { start_time24: { $lt: endTime24 } },
            ],
          },
          {
            $and: [
              { end_time24: { $gt: startTime24 } },
              { end_time24: { $lt: endTime24 } },
            ],
          },
          { start_time24: startTime24 },
          { end_time24: endTime24 },
        ],
      });
      let checkavailability = await ReservationModel.countDocuments({
        employee: employee,
        reservation_date: reservation_date,
        $or: [
          {
            $and: [
              { start_time24: { $gt: startTime24 } },
              { start_time24: { $lt: endTime24 } },
            ],
          },
          {
            $and: [
              { end_time24: { $gt: startTime24 } },
              { end_time24: { $lt: endTime24 } },
            ],
          },
          { start_time24: startTime24 },
          { end_time24: endTime24 },
        ],
        status: { $nin: ["Cancelled", "Expired"] },
      });
      console.log('checkavailability',checkavailability,'absentEmployees',absentEmployees)
      if (checkavailability > 0 || absentEmployees > 0) {
        return res.status(200).json({
          status: "empty",
          message: "Employee not avaialable. choose another employee.",
        });
      } else {
        let employeeRecord = await EmployeeModel.findById(employee);

        employeeRecord.booking_count = employeeRecord.booking_count + 1;
        await employeeRecord.save();
      }
    } else {
      console.log('employee none');
      // get available employee records
      const absentEmployees = await AbsenceModel.find({
        date: reservation_date,
        business_owner : owner._id,
        $or: [
          {
            $and: [
              { start_time24: { $gt: startTime24 } },
              { start_time24: { $lt: endTime24 } },
            ],
          },
          {
            $and: [
              { end_time24: { $gt: startTime24 } },
              { end_time24: { $lt: endTime24 } },
            ],
          },
          { start_time24: startTime24 },
          { end_time24: endTime24 },
        ],
      }).distinct("employee"); // returns array of employee IDs
      
      console.log('absentEmployees',absentEmployees);
      const employees = await EmployeeModel.find({
        $and: [
          { _id: { $nin: absentEmployees } },
          { business_owner : owner._id, },
        ],
      });
      // console.log('employees',employees);return;
      // get available employee records
      let assignedEmployee = null;

      for (let index = 0; index < employees.length; index++) {
        const e = employees[index];
        let availableEmployee = await ReservationModel.findOne({
          employee: employees[index]._id,
          business_owner : owner._id,
          reservation_date: reservation_date,
          $or: [
            {
              $and: [
                { start_time24: { $gt: startTime24 } },
                { start_time24: { $lt: endTime24 } },
              ],
            },
            {
              $and: [
                { end_time24: { $gt: startTime24 } },
                { end_time24: { $lt: endTime24 } },
              ],
            },
            { start_time24: startTime24 },
            { end_time24: endTime24 },
          ],
          status: { $nin: ["Cancelled", "Expired"] },
        });
        // console.log('availableEmployee',availableEmployee);
        if (!availableEmployee) {
          // console.log('no reservation',e);return;
          assignedEmployee = e;
          break;
        }
      }
      if (assignedEmployee === null) {
        return res.status(200).json({
          status: "empty",
          message: "No avaialble employee for the selected slot",
        });
      }
      
      req.body.employee = assignedEmployee._id;
      assignedEmployee.booking_count =
        (assignedEmployee.booking_count || 0) + 1;
      await assignedEmployee.save();
    }
    req.body.staus = owner.automatic_booking == 1 ? "Accepted" : "Pending";
    req.body.booked_date = Date.now();
    // console.log('body',req.body);return;
    let reservations = new ReservationModel(req.body);
    await reservations.save();

    owner.total_booking = owner.total_booking + 1;
    await owner.save();
    return res.status(200).json({
      status: "true",
      message: "Booking Successfully!",
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.bookingReschedule = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  try {
    let updatedData = {};
    updatedData.reservation_date = format(
      new Date(req.body.reservation_date),
      "yyyy-MM-dd"
    );
    updatedData.start_time = req.body.start_time;
    updatedData.end_time = addMinutesToTime(req.body.start_time, account.duration);

    let getbooking = await ReservationModel.findOne({_id : req.body.booking_id});

    const startTime24 = dayjs(updatedData.start_time, "h:mma").format("HH:mm");
    const endTime24 = dayjs(updatedData.end_time, "h:mma").format("HH:mm");4
    updatedData.end_time24 = endTime24;
    updatedData.start_time24 = startTime24;
    console.log('updatedData',updatedData,'startTime24',startTime24,'endTime24',endTime24);
    const absentEmployees = await AbsenceModel.countDocuments({
        employee: getbooking.employee,
        date: updatedData.reservation_date,
        $or: [
          {
            $and: [
              { start_time24: { $gt: startTime24 } },
              { start_time24: { $lt: endTime24 } },
            ],
          },
          {
            $and: [
              { end_time24: { $gt: startTime24 } },
              { end_time24: { $lt: endTime24 } },
            ],
          },
          { start_time24: startTime24 },
          { end_time24: endTime24 },
        ],
      });
      let checkavailability = await ReservationModel.countDocuments({
        employee: getbooking.employee,
        reservation_date: updatedData.reservation_date,
        $or: [
          {
            $and: [
              { start_time24: { $gt: startTime24 } },
              { start_time24: { $lt: endTime24 } },
            ],
          },
          {
            $and: [
              { end_time24: { $gt: startTime24 } },
              { end_time24: { $lt: endTime24 } },
            ],
          },
          { start_time24: startTime24 },
          { end_time24: endTime24 },
        ],
        status: { $nin: ["Cancelled", "Expired"] },
      });
      if (checkavailability > 0 || absentEmployees > 0) {
        return res.status(404).json({
          status: "empty",
          message: "Employee not avaialable. choose another employee.",
        });
      } 

    const changestatus = await ReservationModel.findByIdAndUpdate(
      req.body.booking_id,
      updatedData,
      { new: true } // Return the updated document
    );
    return res.status(200).json({
      status: "true",
      data: changestatus,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.bookingRequirements = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  let given_date = format(new Date(Date.now()), 'yyyy-MM-dd');
  if(req.body.date){
    given_date = format(new Date(req.body.date), 'yyyy-MM-dd');
  }
  console.log('given_date',given_date)
  // const records = await EmployeeModel.find({ business_owner: account._id });

  const records = await EmployeeModel.find({
  business_owner: account._id,
  $or: [{ deleted_at: null }, { deleted_at: { $gt: given_date } }],
});
  console.log('accountdetails',account)
  const timeSlots = getTimeSlots(
    account.opening_time,
    account.closing_time,
    account.duration
  );

  return res.status(200).json({
    status: "true",
    time_slots: timeSlots,
    employees: records,
  });
};

async function getavailableemployee(date, startTime, endTime) {
  const absentEmployees = await EmployeeAbsenceLog.find({
    date: date,
    $or: [
      {
        start_time: { $lt: endTime },
        end_time: { $gt: startTime },
      },
    ],
  }).distinct("employee"); // returns array of employee IDs

  // Step 2: Find employees who are NOT in that list
  const availableEmployees = await Employee.find({
    _id: { $nin: absentEmployees },
  });

  res.json(availableEmployees);
}
