const path = require("path");
const multer = require("multer");

const UserModel = require("../models/Users");
const ClientModel = require("../models/Finalclients");
const SettingModel = require("../models/Settings");
const HelpModel = require("../models/Helps");
const ReservationModel = require("../models/Reservations");

const sendMail = require("./mailController");
// const { ListElementSupport } = require("https://cdn.ckeditor.com/typings/ckeditor5.d.ts");

exports.index = async function (req, res) {
  let { companyname } = req.params;

  let account = await UserModel.findOne({ company_name: companyname });
  let setting = await SettingModel.findOne({});

  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  return res.status(200).json({
    status: "true",
    message: "URL Access successfully",
    settings: {
      title:
        account.language == "fr"
          ? setting.title_fr
          : account.language == "ro"
          ? setting.title_ro
          : setting.title,
      description:
        account.language == "fr"
          ? setting.description_fr
          : account.language == "ro"
          ? setting.description_ro
          : setting.description,
    },
    data: account,
  });
};

exports.login = async function (req, res) {
  let { companyname } = req.params;
  let { email, password, phone_number } = req.body;

  if (!password) {
    return res.status(400).json({
      status: "false",
      message: "Missing required fields",
    });
  }

  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL invalid",
    });
  }
  let final_account = await ClientModel.findOne({ email: email });
  if (!final_account) {
    final_account = await ClientModel.findOne({ phone_number: phone_number });
  }

  if (!final_account) {
    return res.status(404).json({
      status: "false",
      message: "email or phonenumber incorrect",
    });
  }
  if (final_account.password === password) {
    return res.status(200).json({
      status: "true",
      message: "Login Successfully!",
      accountdetail: final_account,
    });
  } else {
    return res.status(404).json({
      status: "false",
      message: "password not matched",
    });
  }
};

exports.register = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }

  let { name, email, password, phone_number } = req.body;

  if (!name || !phone_number || !email || !password) {
    return res.status(400).json({
      status: "false",
      message: "Missing required fields",
    });
  }

  req.body.business_owner = account._id;
  req.body.company_name = account.company_name;
  req.body.created_at = Date.now();

  const newAccount = new ClientModel(req.body);

  try {
    const accountdetails = await newAccount.save();
    return res.status(201).json({
      status: "true",
      message: "Final client successfully successfully",
      data: accountdetails,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.forgotpassword = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }

  let { email } = req.body;
  let client = ClientModel.findOne({ email: email });
  if (!client) {
    return res.status(400).json({
      status: "false",
      message: "Account not found",
    });
  } else {
    return res.status(200).json({
      status: "true",
      message: "Password changed successfully",
    });
  }
};

exports.profile = async function (req, res) {
  let { companyname } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  let { user_id } = req.body;
  let client = await ClientModel.findOne({ _id: user_id });
  return res.status(200).json({
    status: "true",
    data: client,
    requirements: account.requirements,
  });
};

exports.editprofile = async function (req, res) {
  let { companyname,id } = req.params;
  let account = await UserModel.findOne({ company_name: companyname });
  if (!account) {
    return res.status(404).json({
      status: "false",
      message: "URL not found",
    });
  }
  try {
    
    const updatedUser = await ClientModel.findByIdAndUpdate(
      id,
      { $set: req.body },
      { new: true, runValidators: true }
    );
    res.status(200).json({
      message: "Profile updated successfully",
      user: updatedUser,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Server error" });
  }
};


exports.getbookings = async function (req, res) {
  try {
    let { type, user_id } = req.body;
    if (!type || !user_id) {
      return res.status(404).json({
        status: "false",
        message: "Missing Parameters",
      });
    }
    let getbookings;
    if (type == "active") {
      getbookings = await ReservationModel.find({
        final_client: user_id,
        status: { $in: ["Pending", "Accepted"] },
      }).populate("employee");
    } else {
      getbookings = await ReservationModel.find({
        final_client: user_id,
        // status: "Completed",
        status: { $in: ["Cancelled", "Completed"] },
      }).populate("employee");
    }
    if (!getbookings) {
      return res.status(404).json({
        status: "false",
        message: "Bookings not found",
      });
    }

    res.status(200).json({
      bookings: getbookings,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Server error" });
  }
};
