const FCM = require("fcm-node");
const apn = require("apn");
const admin = require("firebase-admin");
const serviceAccount = require("../config/bring-the-noise-firebase-adminsdk-e33lp-30f0428500.json");
if (!admin.apps.length) {
  admin.initializeApp({
    credential: admin.credential.cert(serviceAccount),
    databaseURL: "https://bring-the-noise-default-rtdb.firebaseio.com"
  });
}

/* MODELS */
let UserDevices = require("../models/deviceModel");
let settingModel = require("../models/settingModel");

/* COMPOSE SINGLE PUSH NOTIFICATIONS */
sendNotification = function (member, data) {
  UserDevices.findOne({
    user_id: member
  }).exec(function (err, chatdevices) {
    if (chatdevices && !err) {
      if (chatdevices.device_type == "1") {
        notifyAndroid(chatdevices.device_token, data, "1");
      } else {
        let mode = chatdevices.device_mode;
        let prod_tokens = [];
        prod_tokens.push(chatdevices.fcm_token);
        let wrap_message = {};
        wrap_message.notification_data = data;
        console.log("notify text pavithra4")
        notifyiOS(prod_tokens, wrap_message, data);
      }
    } else {
      console.log("chatdevices ERR: ", err);
    }
  });
};

/* COMPOSE MULTI PUSH NOTIFICATIONS */
sendNotifications = function (members, data) {
  UserDevices.find({
    device_type: "1"
  }).where("user_id").in(members).exec(function (err, records) {
    let devicetokens = [];
    if (records != "" && typeof records != "undefined") {
      records.forEach(function (rec) {
        devicetokens.push(rec.device_token);
      });
      notifyAndroid(devicetokens, data, "2");
    }
  });

  let wrap_message = {};
  wrap_message.notification_data = data;
  UserDevices.find({
    device_type: "0"
  }).where("user_id").in(members).exec(function (err, records) {
    let sandbox_tokens = [];
    let prod_tokens = [];
    if (records != "" && typeof records != "undefined") {
      records.forEach(function (rec) {
        let mode = rec.device_mode;
        if (mode == "1") {
          prod_tokens.push(rec.fcm_token);
        } else {
          sandbox_tokens.push(rec.fcm_token);
        }
      });
      if (prod_tokens.length > 0) {
        notifyiOS(prod_tokens, wrap_message, data);
      }
      if (sandbox_tokens.length > 0) {
        notifyiOS(sandbox_tokens, wrap_message, data);
      }
    }
  });
};

/* FCM TEXT PUSH NOTIFICATIONS ANDROID */
const notifyAndroid = function (devicetokens, data, recipients) {
  let settingsQuery = settingModel.findOne({}).limit(1);
  settingsQuery.exec(function (err, settings) {
    if (!err) {
      for (const key in data) {
        if (data.hasOwnProperty(key)) {
          data[key] = String(data[key]);
        }
      }

      let message;
      if (recipients == "1") {
        message = {
          data: data,
          token: devicetokens,
          android: {
            priority: "high"
          }
        };
        admin.messaging().send(message).then(function (response) {
          // console.log('notifyAndroid single recipient response: ', response);
        }).catch(function (error) {
          // console.log('notifyAndroid single recipient error: ', error);
        });
      } else {
        message = {
          data: data,
          tokens: devicetokens,
          android: {
            priority: "high"
          }
        };
        admin.messaging().sendMulticast(message).then(function (response) {
          // console.log("notifyAndroid multiple recipient response: ", response);
        }).catch(function (error) {
          // console.log('notifyAndroid multiple recipient error:', error);
        });
      }
    } else {
      console.log("Error fetching settings: " + err);
    }
  });
};

/* iOS PUSH NOTIFICTIONS */
const notifyiOS = function (devicetokens, wrap_message, data) {
  let settingsQuery = settingModel.findOne({}).limit(1);
  settingsQuery.exec(function (err, settings) {
    if (!err) {
      for (const key in data) {
        if (data.hasOwnProperty(key)) {
          data[key] = String(data[key]);
        }
      }

      let aps = {
        alert: {},
        sound: "default",
        contentAvailable: true
      };

      if (["follow", "comment", "mention", "like", "gift", "streaminvitation", "followeronlive", "videoposted", "followerpostvideo", "useronlive"].includes(data.scope)) {
        aps.alert.title = data.user_name;
        aps.alert.body = data.message;
      } else {
        if (data.msg_type == "text") {
          const key = "crypt@123";
          const cryptLib = require('@skavinvarnan/cryptlib');
          const decryptedString = cryptLib.decryptCipherTextWithRandomIV(data.message, key);
          aps.alert.body = decryptedString;
          aps.alert.title = data.user_name;
        }

        if (data.msg_type == "missed") {
          aps.alert.body = 'Missed call';
          aps.alert.title = data.user_name;
        }

        if (data.msg_type == "image") {
          aps.alert.body = 'sent an image';
          aps.alert.title = data.user_name;
        }

        if (data.msg_type == "gif") {
          aps.alert.body = 'sent a gif';
          aps.alert.title = data.user_name;
        }

        if (data.msg_type == "audio") {
          aps.alert.body = 'sent an audio';
          aps.alert.title = data.user_name;
        }
      }

      let wrap_messagenew = JSON.stringify(wrap_message.notification_data);

      let message = {
        data: {
          notification_data: wrap_messagenew
        },
        notification: {
          title: aps.alert.title,
          body: aps.alert.body
        },
        apns: {
          payload: {
            aps: aps
          },
          headers: {
            "apns-priority": "10"
          }
        }
      };

      if (Array.isArray(devicetokens) && devicetokens.length > 1) {
        sendNotificationToTokens(devicetokens, message);
      } else if (devicetokens.length === 1) {
        message.token = devicetokens[0];
        admin.messaging().send(message).then(function (response) {
          // console.log('notifyiOS single recipient response:', response);
        }).catch(function (error) {
          if (error.code === 'messaging/registration-token-not-registered') {
            // console.log('Invalid registration token:', devicetokens[0]);
          } else {
            // console.log('notifyiOS single recipient error:', error);
          }
        });
      } else {
        console.log("No device tokens provided");
      }
    } else {
      console.error("Error fetching settings: ", err);
    }
  });
};

function sendNotificationToTokens(devicetokens, message) {
  const validTokens = devicetokens.filter(token => token);

  validTokens.forEach((token, index) => {
    const individualMessage = {
      notification: message.notification || null,
      data: message.data || null,
      token: token
    };
    admin.messaging().send(individualMessage).then((response) => {
       console.log(`Successfully sent notification to device ${index + 1}: `, response);
    }).catch((error) => {
       console.error(`Error sending notification to device ${index + 1}: `, error.message);
    });
  });
}

/* iOS PUSH NOTIFICTIONS */
callnotifyiOS = function (devicetoken, message_data, mode) {
  console.log("***");
  console.log("callNotification");
  console.log("***");
  let settingsQuery = settingModel.findOne({}).limit(1);
  settingsQuery.exec(function (err, appsettings) {
    let apns_passphrase = appsettings.voip_passpharse;
    if (typeof apns_passphrase != "undefined" && apns_passphrase != null) {
      let options = {};
      options.cert = __dirname + "/../../../img" + "/voip/cert.pem";
      options.key = __dirname + "/../../../img" + "/voip/key.pem";
      options.passphrase = apns_passphrase;
      options.production = false;
      if (mode == "1") {
        options.production = true;
      }
      let apnProvider = new apn.Provider(options);
      let notification = new apn.Notification();
      notification.payload = message_data;
      notification.badge = 1;
      notification.alert = message_data.message;
      apnProvider.send(notification, devicetoken).then(result => {
        console.log("iOS call Pushnotification log" + JSON.stringify(result));
      });
    }
  });
};

/* COMPOSE SINGLE PUSH NOTIFICATIONS */
callNotification = function (member, data) {
  console.log("***");
  console.log("callNotification");
  console.log("***");
  UserDevices.findOne({
    user_id: member
  }).exec(function (err, chatdevices) {
    if (chatdevices && !err) {
      if (chatdevices.device_type == "1") {
        notifyAndroid(chatdevices.device_token, data, "1");
      } else {
        let mode = chatdevices.device_mode;
        let wrap_message = {};
        wrap_message.notification_data = data;
        callnotifyiOS(chatdevices.voip_token, wrap_message, mode);
      }
    } else {
      console.log('videocall not working');
    }
  });
};

/* COMPOSE MULTI PUSH NOTIFICATIONS */
callNotifications = function (members, data) {
  UserDevices.find({
    device_type: "1"
  }).where("user_id").in(members).exec(function (err, records) {
    let devicetokens = [];
    if (records != "" && typeof records != "undefined") {
      records.forEach(function (rec) {
        devicetokens.push(rec.device_token);
      });
      notifyAndroid(devicetokens, data, "2");
    }
  });

  let wrap_message = {};
  wrap_message.notification_data = data;
  UserDevices.find({
    device_type: "0"
  }).where("user_id").in(members).exec(function (err, records) {
    let sandbox_tokens = [];
    let prod_tokens = [];
    if (records != "" && typeof records != "undefined") {
      records.forEach(function (rec) {
        let mode = rec.device_mode;
        if (mode == "1") {
          prod_tokens.push(rec.device_token);
        } else {
          sandbox_tokens.push(rec.device_token);
        }
      });
      if (prod_tokens.length > 0) {
        callnotifyiOS(prod_tokens, wrap_message, "1");
      }
      if (sandbox_tokens.length > 0) {
        callnotifyiOS(sandbox_tokens, wrap_message, "0");
      }
    }
  });
};

module.exports = {
  sendNotification: sendNotification,
  sendNotifications: sendNotifications,
  callNotification: callNotification,
  callNotifications: callNotifications
};
